@@ -26,7 +26,7 @@
 
 BigDecimal::BigDecimal()
 {
-    _val = Chain(0);
+    _val = Chain("00");
     _scale=1;
     _isPositive = true;
 }
@@ -62,27 +62,28 @@ BigDecimal::BigDecimal(const Chain& val)
     Tokenizer tok(_val, Chain("."));
     Chain left, right;
     tok.nextToken(left);
-    tok.nextToken(right);
-
-    /*
-    right = right.truncRight("0");
-    if ( right.length() < 2 )
-	right = Chain("0");
-    */
-    
-    _scale = right.length() > 0 ? right.length() - 1 : 0;
+    tok.nextToken(right);    
 
     left = left.truncLeft("0");
     if ( left.length() < 2 )
-	left = Chain("0");
-
-    /*
-    cout << "Left = " << left << endl;
-    cout << "Right = " << right << endl;
-    cout << "Scale = " << _scale << endl;
-    */
-    _val = left + right;
+    {
+	left = Chain("0");	
+    }
     
+    if ( right.length() < 2 )
+    {
+	right = Chain("0");
+    }
+    
+    _scale = right.length() - 1; //  > 0 ? right.length() - 1 : 1;
+	       
+    // cout << "Left = " << left << endl;
+    // cout << "Right = " << right << endl;
+    // cout << "Scale = " << _scale << endl;
+    
+    _val = left + right;
+
+    // cout << "Val=" << _val << endl;
 }
 
 BigDecimal::BigDecimal(const Chain& val, int scale)
@@ -107,8 +108,19 @@ BigDecimal::BigDecimal(const Chain& val, int scale)
     }
     
     _scale = scale;
-}
 
+    
+    if ( _val.length() - 1  <= _scale )
+    {
+	int j = _val.length() - 1;
+		
+	for ( int i = j; i <= _scale ; i++)
+	{
+	    _val = "0" + _val;
+	}
+    }
+    // cout << "Val=" << _val << endl;
+}
 
 BigDecimal::~BigDecimal()
 {
@@ -116,7 +128,7 @@ BigDecimal::~BigDecimal()
 
 BigDecimal BigDecimal::abs() const
 {
-    return BigDecimal(_val);
+    return BigDecimal(_val, _scale);
 }
 
 bool BigDecimal::isPositive() const
@@ -213,7 +225,7 @@ BigDecimal BigDecimal::mul(const BigDecimal& d) const
 	i2.negate();
     
     BigInteger i3 =  i1 * i2;
-
+    
     return BigDecimal(i3.toChain(), _scale + d._scale); 
     
 }
@@ -232,8 +244,9 @@ BigDecimal BigDecimal::div(const BigDecimal& d) const
     }
 
     if ( i == v1.length() - 1 )
+    {
         return BigDecimal();
-	 
+    }	 
     int maxScale;
     int corScale;
     
@@ -287,15 +300,25 @@ BigDecimal BigDecimal::div(const BigDecimal& d) const
 	}
     }
     
+    // cout << "V = " << v << endl;
+    // cout << "MaxScale = " << maxScale << endl;
     return BigDecimal(v, maxScale); 
 }
 
-BigDecimal BigDecimal::scaleTo(int scale) const
+BigDecimal BigDecimal::scaleTo(int scale, RoundMode mode) const
 {
+
+    // cout << "Scale=" << _scale << endl;
+    // cout << "Val=" << _val << endl;
+    
     BigDecimal scaleTarget;
     
     if ( scale == _scale )
 	return *this;
+    else if ( _val.truncLeft("0") == Chain() )
+    {
+	scaleTarget = BigDecimal(Chain("0"), scale);
+    }
     else if ( scale > _scale )
     {
 	Chain val = _val;
@@ -308,10 +331,23 @@ BigDecimal BigDecimal::scaleTo(int scale) const
 	scaleTarget = BigDecimal(val, scale);
     }
     else
-    {	
-	Chain roundVal = _val.subChain(1, _val.length() - ( _scale - scale ) -1);
-	Chain roundCheck =  _val.subChain(_val.length() - scale - 1, _val.length() - scale - 1);
-	if ( roundCheck.asInteger() >= 5 )
+    {
+	int deltaScale = _scale - scale;
+	
+	Chain roundVal = _val.subChain(1, _val.length() - deltaScale -1);	
+	Chain roundCheck =  _val.subChain(_val.length() - deltaScale , _val.length() - deltaScale);
+
+	// cout << "RoundCheck=" << roundCheck << endl;
+
+	bool roundUp=true;
+
+	if ( mode == DOWN
+	     || ( roundCheck.asInteger() <= 5 && mode == HALFDOWN ) )
+	{
+	    roundUp=false;
+	}
+	
+	if ( roundUp ) // roundCheck.asInteger() >= 5 )
 	{
 	    Chain delta = Chain("0");
 	    for ( int i=0; i<scale-1; i++ )
@@ -343,25 +379,17 @@ int BigDecimal::length() const
 
 Chain BigDecimal::toChain() const
 {
+    // cout << "Val=" << _val << endl;
+    // cout << "Scale=" << _scale << endl;
+    
     Chain s;
 
     if ( _isPositive == false )
 	s = Chain("-");
-
-    Chain val = _val;
     
-    if ( val.length() - 1  <= _scale )
-    {
-	int j = val.length() - 1;
-	for ( int i = j; i <= _scale ; i++)
-	{
-	    val = "0" + val;
-	}
-    }
-    
-    s +=  val.subChain(1, val.length() - _scale - 1) 
+    s +=  _val.subChain(1, _val.length() - _scale - 1) 
 	+ Chain(".")
-	+ val.subChain(val.length() - _scale, val.length()); 	
+	+ _val.subChain(_val.length() - _scale, _val.length()); 	
 
     return s;
 }
