@@ -189,40 +189,55 @@ Screen2::Message::~Message()
 	delwin(_msgwin);
 }
 
-void Screen2::Message::setInfo(const Chain& title, const Chain& msg)
+void Screen2::Message::setInfo(const Chain& title, const Chain& msg, int height)
 {
     _title = title;
     _msg = msg;
     
     _width = _title.length();
-    _height=0; 
-    
+
+    if ( height > 0 )
+	_height=height; 
+    else
+	_height=0;
+  
     Tokenizer t(_msg, Chain("\n"));
     Chain l;
+    _msgHeight = 0;
     while ( t.nextToken(l))
     {
-	_height++;
+	if ( height == 0 )
+	    _height++;
 	if ( l.length() > _width )
 	    _width = l.length();
+	_msgHeight++;
     }
     
     _height+=4;
+    
     _width+=3;
-
+    _offset=0;
 }
 
 void Screen2::Message::show(bool doRec)
 {
 
-    _pParent->show(doRec);
+    if ( doRec )
+	_pParent->show(doRec);
     
     int startx, starty;
     calcOffset(_pParent->getWindow(), starty, startx);
 
-    if ( _msgwin == 0 )    
-	_msgwin = newwin( _height, _width, starty, startx);
+
+    // since message window width and height may have changed we allocate new window
+    if ( _msgwin != 0 )
+	delwin(_msgwin);
+    
+    _msgwin = newwin( _height, _width, starty, startx);
 
     wclear(_msgwin);
+
+    keypad(_msgwin, TRUE);
   
     wattron(_msgwin, A_BOLD);
     mvwprintw(_msgwin, 1, 2, "%s", (char*)_title);
@@ -231,11 +246,16 @@ void Screen2::Message::show(bool doRec)
 
     Tokenizer t(_msg, Chain("\n"));
     int lno=3;
+    int lineNo = 0;
     Chain l;
     while ( t.nextToken(l))
     {
-	mvwprintw(_msgwin, lno, 2, "%s", (char*)l);
-	lno++;
+	if ( lineNo >= _offset && lno < _height - 1)
+	{
+	    mvwprintw(_msgwin, lno, 2, "%s", (char*)l);
+	    lno++;
+	}
+	lineNo++;
     }
 
     wrefresh(_msgwin);
@@ -243,15 +263,29 @@ void Screen2::Message::show(bool doRec)
 
 void Screen2::Message::handleKey(int c)
 {
-    if ( c == KEY_RESIZE )
+    switch ( c )
     {
+    case KEY_RESIZE:
 	 int yoff, xoff;
 	 calcOffset(_pParent->getWindow(), yoff, xoff);
 	 mvwin(_msgwin, yoff, xoff);
 	_pParent->handleKey(c);
+	return;
+    case KEY_UP:
+	if ( _offset > 0 )
+	    _offset--;
+	_pNext = this;
+	// setStatus("Offset = " + Chain(_offset));
+	return;
+    case KEY_DOWN:
+	if ( _offset < _msgHeight - _height + 4 )
+	    _offset++;
+	_pNext = this;
+	// setStatus("Offset = " + Chain(_offset));
+	return;
+    default:
+	_pNext = _pParent;
     }
-    else
-	_pNext = _pParent;    
 }
 
 WINDOW* Screen2::Message::getWindow()
@@ -270,8 +304,7 @@ Screen2::Select::Select(Screen2::Panel *pParent) : Panel(pParent)
 }
 
 Screen2::Select::~Select()
-{
-    if ( _selectwin != 0 )
+{    if ( _selectwin != 0 )
 	delwin(_selectwin);
     delete _pError;
 }
@@ -448,13 +481,14 @@ void Screen2::Grid::setSchema(const Chain& title, const ListT<Chain>& gridSchema
     _schema = gridSchema;
     _colorMap = colorMap;
     
-    _numCol=0;
+    _numCol=2;
     Chain *pA = _schema.First();
     while ( pA )
     {
 	Chain attr, attrLen;
 	getKeyValue(*pA, attr, attrLen);
-	_numCol = _numCol + attrLen.asInteger();	    
+	
+	_numCol = _numCol + attrLen.truncLeft("r").truncLeft("l").asInteger();	    
 	pA = _schema.Next();
     }
     _initHeight = height;
@@ -609,12 +643,19 @@ void Screen2::Grid::show(bool doRec)
     Chain *pA = _schema.First();
     while ( pA )
     {
-	Chain attr, attrLen;
-	getKeyValue(*pA, attr, attrLen);
+	Chain attr, lenInfo;
+	getKeyValue(*pA, attr, lenInfo);
+
+	int attrLen = lenInfo.truncLeft("r").truncLeft("l").asInteger();
 	
-	mvwprintw(_gridwin, rowno, colno, "%s", (char*)attr);
-	
-	colno = colno + attrLen.asInteger();
+	if ( lenInfo[0] == 'l' )
+	    mvwprintw(_gridwin, rowno, colno, "%s", (char*)attr);
+	else
+	{
+	    int offset = attrLen - attr.length();
+	    mvwprintw(_gridwin, rowno, colno+offset, "%s", (char*)attr);
+	}
+	colno = colno + attrLen;
 	
 	pA = _schema.Next();
     }
@@ -674,14 +715,25 @@ void Screen2::Grid::refreshRow(int tabRow, int rowno)
 	    }
 	}
 	
-	Chain attr, attrLen;
-	getKeyValue(*pA, attr, attrLen);
+	Chain attr, lenInfo;
+	getKeyValue(*pA, attr, lenInfo);
+
+	int attrLen = lenInfo.truncLeft("r").truncLeft("l").asInteger();
+
+	// for ( int i=0; i < attrLen ; i++)
+	//     mvwprintw(_gridwin, rowno, colno+i, "%s", " ");
+		    
+	if ( lenInfo[0] == 'l' )
+	{	    
+	    mvwprintw(_gridwin, rowno, colno, "%s", (char*)*pVal);
+	}
+	else
+	{
+	    int offset = attrLen - pVal->length();
+	    mvwprintw(_gridwin, rowno, colno+offset, "%s", (char*)*pVal);	    
+	}
 	
-	for ( int i=0; i < attrLen.asInteger() ; i++)
-	    mvwprintw(_gridwin, rowno, colno+1, "%s", " ");
-	mvwprintw(_gridwin, rowno, colno, "%s", (char*)*pVal);
-	
-	colno = colno + attrLen.asInteger();
+	colno = colno + attrLen;
 	
 	wcolor_set(_gridwin, rowColor, NULL);
 	
@@ -1332,6 +1384,7 @@ void Screen2::Form::handleKey(int c)
 		if ( type == Chain(INPUT_TYPE_STRING)
 		     && ( isalnum (c)
 			  || c == '_'
+			  || c == '|'
 			  || c == '-'
 			  || c == '/'
 			  || c == '.'
@@ -1515,15 +1568,17 @@ Screen2::Panel* Screen2::Form::FormSelect::onSelect(int selected)
     return _pParent;
 }
 
+
 ////////////////////////////////
 // Screen2::Attribute methods //
 ////////////////////////////////
 
-Screen2::Attribute::Attribute(Screen2::Panel *pParent, bool isVertical) : Panel(pParent) 
-{ 
-    _isVertical = isVertical;
+Screen2::Attribute::Attribute(Screen2::Panel *pParent, Layout layout) : Panel(pParent)
+{
+    _layout = layout;
     for ( int i=0; i < MAX_ATTR_WIN; i++)
-	_attrwin[i]=0;
+        _attrwin[i]=0;
+    _visibleIdx=0;
 }
 
 Screen2::Attribute::~Attribute()
@@ -1533,94 +1588,149 @@ Screen2::Attribute::~Attribute()
 	    delwin(_attrwin[i]);
 }
 
-void Screen2::Attribute::setAttrList(const Chain& title, const ListT< ListT<Chain> >& attrList)
+void Screen2::Attribute::setAttrList(const ListT<Chain>& titleList, const ListT< ListT<Chain> >& attrList)
 {
-    _title = title;    
+    _titleList = titleList;
     _attrList = attrList;
 }
 
 void Screen2::Attribute::show(bool doRec)
 {
-    if ( doRec )
-	_pParent->show(doRec);
-        
-    int widx=0;
+    // we show recursive in any case to cleanup windows for OVERLAY layout
+    _pParent->show(true);
+    
     int xpos=1;
     int ypos=2;
     
-    ListT<Chain> *pKVL = _attrList.First();
-    while ( pKVL )
+    if ( _attrList.Size() == 0 )
+	return;
+    
+    if ( _layout == OVERLAY )
     {
-	int maxKeyLen=0;
-	int maxValLen=0;
-	Chain *pKV = pKVL->First();
-	while ( pKV )
+	int widx=_visibleIdx;
+	
+	if ( _attrwin[widx] != 0 )
 	{
-	    Chain key, value;
-
-	    Tokenizer t(*pKV, Chain(VALSEPTOKEN));
-	    t.nextToken(key);
-	    t.nextToken(value);
-	    
-	    if ( key.length() > maxKeyLen )
-		maxKeyLen = key.length();
-	    if ( value.length() > maxValLen )
-		maxValLen = value.length();
-	    pKV = pKVL->Next();
+	    wclear(_attrwin[widx]);
+	    delwin(_attrwin[widx]);
 	}
 	
-	if ( _attrwin[widx] == 0 )
-	    _attrwin[widx] = newwin(pKVL->Size() + 2, maxKeyLen + maxValLen + 3, ypos, xpos);
-
-	wclear(_attrwin[widx]);
-	
-	noecho();		
-	keypad(_attrwin[widx], TRUE);
-	
-	box(_attrwin[widx], 0, 0);
-	
-	wcolor_set(_attrwin[widx], COL_TITLE, NULL);
-		
-	int rowno=1;
-	pKV = pKVL->First();
-	while ( pKV )
-	{
-	    Chain key, value;
-
-	    Tokenizer t(*pKV, Chain(VALSEPTOKEN));
-	    t.nextToken(key);
-	    t.nextToken(value);
-	    
-	    int colno=2;
-	    
-	    wcolor_set(_attrwin[widx], COL_TITLE, NULL);
-	    mvwprintw(_attrwin[widx], rowno, colno, "%s", (char*)key);
-
-	    colno += maxKeyLen;
-	    
-	    wcolor_set(_attrwin[widx], COL_CONTENT, NULL);
-	    mvwprintw(_attrwin[widx], rowno, colno, "%s", (char*)value);
-	    
-	    rowno++;
-	    pKV = pKVL->Next();
-	}
-
-	if ( _isVertical )
-	    ypos += pKVL->Size() + 2;
-	else
-	    xpos += maxKeyLen + maxValLen + 3;
-
+	_attrwin[widx] = createWindow(xpos, ypos, _titleList[widx], &_attrList[widx]);
 	
 	wrefresh(_attrwin[widx]);
-	widx++;
-	pKVL = _attrList.Next();
+	wrefresh(stdscr);
+	
     }
+    else
+    {
+	int widx=0;
+	ListT<Chain> *pKVL = _attrList.First();
+	while ( pKVL )
+	{
+	    _attrwin[widx] = createWindow(xpos, ypos, _titleList[widx], pKVL);
+	    	    
+	    int maxKeyLen=0;
+	    int maxValLen=0;
+	    Chain *pKV = pKVL->First();
+	    while ( pKV )
+	    {
+		Chain key, value;
+		
+		Tokenizer t(*pKV, Chain(VALSEPTOKEN));
+		t.nextToken(key);
+		t.nextToken(value);
+		
+		if ( key.length() > maxKeyLen )
+		    maxKeyLen = key.length();
+		if ( value.length() > maxValLen )
+		    maxValLen = value.length();
+		pKV = pKVL->Next();
+	    }
+	    
+	    
+	    if ( _layout == VERTICAL )
+		ypos += pKVL->Size() + 2;
+	    else
+		xpos += maxKeyLen + maxValLen + 3;
+	    
+	    
+	    wrefresh(_attrwin[widx]);
+	    wrefresh(stdscr);
+	    
+	    widx++;
+	    pKVL = _attrList.Next();
+	}
+    }
+}
+
+WINDOW* Screen2::Attribute::createWindow(int xpos, int ypos, const Chain& title, ListT<Chain> *pAttrList)
+{
+    
+    WINDOW* attrwin;
+    
+    int maxKeyLen=0;
+    int maxValLen=0;
+    Chain *pKV = pAttrList->First();
+    while ( pKV )
+    {
+	Chain key, value;
+	
+	Tokenizer t(*pKV, Chain(VALSEPTOKEN));
+	t.nextToken(key);
+	t.nextToken(value);
+	
+	if ( key.length() > maxKeyLen )
+	    maxKeyLen = key.length();
+	if ( value.length() > maxValLen )
+	    maxValLen = value.length();
+	pKV = pAttrList->Next();
+    }
+    
+    attrwin = newwin(pAttrList->Size() + 2, maxKeyLen + maxValLen + 3, ypos, xpos);
+    
+    wclear(attrwin);
+    
+    noecho();
+    keypad(attrwin, TRUE);
+    
+    box(attrwin, 0, 0);
+    
+    wcolor_set(attrwin, COL_TITLE, NULL);
+    wattron(attrwin, A_BOLD);
+    
+    mvwprintw(attrwin, 0, 2, "%s", (char*)title);
+    
+    int rowno=1;
+    pKV = pAttrList->First();
+    while ( pKV )
+    {
+	Chain key, value;
+	
+	Tokenizer t(*pKV, Chain(VALSEPTOKEN));
+	t.nextToken(key);
+	t.nextToken(value);
+	
+	int colno=2;
+	
+	wcolor_set(attrwin, COL_TITLE, NULL);
+	mvwprintw(attrwin, rowno, colno, "%s", (char*)key);
+	
+	colno += maxKeyLen;
+	
+	wcolor_set(attrwin, COL_CONTENT, NULL);
+	mvwprintw(attrwin, rowno, colno, "%s", (char*)value);
+	
+	rowno++;
+	pKV = pAttrList->Next();
+    }
+    
+    return attrwin;
 }
 
 void Screen2::Attribute::handleKey(int c)
 {
     switch(c)
-    {	
+    {
     case KEY_RETURN:
     case KEY_ESC:
     {	
@@ -1632,13 +1742,31 @@ void Screen2::Attribute::handleKey(int c)
 		_attrwin[i]=0;
 	    }
 	}
+	
+	_visibleIdx=0;
+	
 	_pNext = _pParent;
 	return;
     }
-
+    case KEY_LEFT:
+    {
+	if ( _visibleIdx > 0 )
+	    _visibleIdx--;
+	else
+	    _visibleIdx = _attrList.Size() -1;
+	break;
+    }
+    case KEY_RIGHT:
+    {
+	if ( _visibleIdx >= _attrList.Size() -1 )
+	    _visibleIdx = 0;
+	else
+	    _visibleIdx++;
+	break;
+    }
     case KEY_RESIZE:
     {
-	/*
+	
 	for ( int i=0; i < MAX_ATTR_WIN; i++)
 	{
 	    if ( _attrwin[i] != 0 )
@@ -1647,19 +1775,20 @@ void Screen2::Attribute::handleKey(int c)
 		_attrwin[i]=0;
 	    }
 	}
-	*/
+	
 	_pParent->handleKey(c);
 	break;
     }
     }
-    _pNext = this;    
+    _pNext = this;
 }
 
 WINDOW* Screen2::Attribute::getWindow()
 {
-    return _attrwin[0];
+    return stdscr;
 }
 
+
 /////////////////////
 // Screen2 methods //
 /////////////////////
